#!/bin/sh
### BEGIN INIT INFO
# Provides:          SickGear-daemon
# Required-Start:    $local_fs $remote_fs $network
# Required-Stop:     $local_fs $remote_fs $network
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: Start or stop the SickGear-daemon.
### END INIT INFO

PKG_NAME="SickGear"
PKG_DIR="/usr/local/AppCentral/sickgear"
RUN_AS="admin:administrators"
CONFIG_DIR="$PKG_DIR/config"
REPOS_DIR="$PKG_DIR/repository"
PYTHON="/opt/bin/python3"
PID_FILE="$PKG_DIR/$PKG_NAME.pid"
DAEMON="python3 $PKG_DIR/$PKG_NAME/sickgear.py"
DAEMON_OPTS=" --daemon --datadir $PKG_DIR/config --pidfile $PID_FILE"

CreateVenv() {
	cd $PKG_DIR
	ARCH=$(/bin/uname -m)
	if [ "$ARCH" = "x86_64" ]; then
		HEADER="http://bin.entware.net/x64-k3.2/include/include.tar.gz"
	elif [ "$ARCH" = "armv7l" ]; then
		HEADER="http://bin.entware.net/armv7sf-k3.2/include/include.tar.gz"
	elif [ "$ARCH" = "aarch64" ]; then      
        		HEADER="http://bin.entware.net/aarch64-k3.10/include/include.tar.gz"
	else
		HEADER="http://bin.entware.net/x86-k2.6/include/include.tar.gz"
		#check for issue where entware does not have enough space when /opt is a directory in /
		if ! [[ -L /opt && -d /opt ]]; then
			# move opt into volume1 and create link back into /opt
			[ -d /volume1/opt ] || mv /opt /volume1/opt
			[ -d /opt ] && mv /opt/* /volume1/opt && rmdir /opt
			ln -sf /volume1/opt /opt
		fi	
		if [ ! -f /opt/bin/grep ]; then
			# re-run the entware install
			/usr/local/AppCentral/entware/CONTROL/start-stop.sh install_again			
		fi	
	fi
	/bin/echo -n " Creating new virtual environment"
	# Install these if not already present.
	/opt/bin/opkg update
	/opt/bin/opkg install python3
	/opt/bin/opkg install python3-pip
	/opt/bin/opkg install python3-cffi
	/opt/bin/opkg install python3-cryptography
	# install additional tools
	/opt/bin/opkg install unrar
	# Include gcc and headers to allow native compiling.
	/opt/bin/opkg install gcc
	python_version="$($PYTHON --version | sed 's/Python //g')"
	py1="$(echo $python_version | cut -d'.' -f1)"
	py2="$(echo $python_version | cut -d'.' -f2)"
	wget -qO- $HEADER | tar xvz -C /opt/include
	$PYTHON -m pip install --upgrade pip
	$PYTHON -m pip install --user virtualenv
        # create venv without pip due to entware pip issues.
	$PYTHON -m venv --clear --without-pip env
	# copy existing lib packgaes to venev
	cp -rf /opt/lib/python${py1}.${py2}/site-packages/* env/lib/python${py1}.${py2}/site-packages/
        # activate venev
        source env/bin/activate
        # Within it, invoke this well-known script to manually install pip(1) into env
        curl https://bootstrap.pypa.io/get-pip.py | python
        deactivate
        # Reactivate now including the pip(1) command.
        source env/bin/activate
        # And finally, upgrade pip(1) itself  
        python -m pip install --upgrade pip
        deactivate		
}


WriteIni() { #writes a value to a parameter in a section in a file
        local SECTION=$1
        local PARAM=$2
        local VALUE=$3
        local FILE=$4
        /bin/sed -ie "/^\[${SECTION}\]/,/^\[.*\]/s|^\(${PARAM}[ \t]*=[ \t]*\).*$|\1${VALUE}|" ${FILE}
}
                                        
ReadIni() { #reads a value from a parameter in a section in a file
        local SECTION=$1
        local PARAM=$2
        local FILE=$3
        /bin/echo $(/bin/sed -n "/^\[${SECTION}\]/,/^\[.*\]/s/^${PARAM}[ \t]*=[ \t]*\(.*$\)/\1/p" ${FILE})
}

WriteDefaults() { #Write the default values to a bog standard config file (fresh install only)
        /bin/echo -n " Writing default values to the config file..."
        
        local CONFIGFILE=$CONFIG_DIR/config.ini
              
        [ -f $CONFIGFILE ] || /bin/cp $CONFIGFILE.org $CONFIGFILE
                
        WriteIni General web_port 7081 $CONFIGFILE
        WriteIni General web_host 0.0.0.0 $CONFIGFILE
        WriteIni General git_path "/usr/local/bin/git" $CONFIGFILE
        /bin/echo  " Done"
}
                                                                        
CheckPkgRunning() { #Is the PKG already running? if so, exit the script
      [ -f $PID_FILE ] && /sbin/start-stop-daemon -K --test -q -p $PID_FILE -x ${DAEMON} && /bin/echo " $PKG_NAME is already running" && exit 0
      [ -f $PID_FILE ] && /bin/rm -f $PID_FILE
	#ok, we survived so the PKG should not be running
} 

StartPkg(){ #starts the PKG
	/bin/echo -n "Starting $PKG_NAME" 
	GIT_URL="https://github.com/SickGear/SickGear.git"
	[ -d $PKG_DIR/$PKG_NAME/.git ] || /usr/local/bin/git clone $GIT_URL $PKG_DIR/$PKG_NAME
	[ -f $CONFIG_DIR/config.ini ] || WriteDefaults #creates and writes the default config file on a fresh install 
	
	#Switch to virtual env and check for required modules.
	cd $PKG_DIR

	# build venev if it doesn't exist
	[ -f $PKG_DIR/env/bin/activate ] || CreateVenv

	# build new venv if different py version
	python_version="$($PYTHON --version | sed 's/Python //g')"
	venv_version="$(cat env/pyvenv.cfg | grep version | sed 's/version = //g')"
	[ "$python_version" = "$venv_version" ] || CreateVenv

	# activate venv and install requirements
	source env/bin/activate
	
	cd $PKG_DIR/$PKG_NAME 
	/usr/local/bin/git pull
	python -m pip install --upgrade pip
	python -m pip install -r requirements.txt

	# optimizations and known version issues.
	python -m pip install lxml
	python -m pip install regex
	#default version of feedparser has issues. force use of feedparser>=6.0.0.b3.
		#python -m pip install "feedparser>=6.0.0.b3"

	/bin/chown -R $RUN_AS $PKG_DIR
	/bin/chmod -R -t $PKG_DIR # make sure no system bits are set.
	/bin/chmod -R -s $PKG_DIR # make sure no system bits are set.
	/sbin/start-stop-daemon -S -n $PKG_NAME -p $PID_FILE -c $RUN_AS -x ${DAEMON} --${DAEMON_OPTS}
	/bin/echo " Done ($(/bin/date))"
}

ShutdownPkg() { #kills a proces based on a PID in a given PID file
	if [ -f $PID_FILE ] && /sbin/start-stop-daemon -K --test -q -p $PID_FILE -x ${DAEMON}; then
		#grab pid from pid file
		Pid=$(/bin/cat $PID_FILE)
		i=0
		/sbin/start-stop-daemon -K -q -p $PID_FILE -x ${DAEMON}
		/bin/echo -n " Waiting for ${PKG_NAME} to shut down: "
		while [ -d /proc/$Pid ]; do
			sleep 1
			let i+=1
			/bin/echo -n "$i, "
			if [ $i = 45 ]; then
				/bin/echo -n " Tired of waiting, killing ${PKG_NAME} now"
				/bin/kill -9 $Pid
				/bin/rm -f $PID_FILE
				/bin/echo " Done ($(/bin/date))"
				exit 1
			fi
		done
		/bin/rm -f $PID_FILE
		/bin/echo "Done ($(/bin/date))"
	else
		/bin/echo "${PKG_NAME} is not running? ($(/bin/date))"
		if [ -f $PID_FILE ]; then
			/bin/rm -f $PID_FILE
		fi
	fi
}

case "$1" in
	start)
		/bin/echo "$PKG_NAME prestartup checks... ($(/bin/date))"	
		CheckPkgRunning  #Check if the PKG is not running, else exit
		StartPkg	 #Start the PKG... (finaly ;) )
	;;
	stop)
		/bin/echo "Shutting down ${PKG_NAME} at $(/bin/date)... "
		ShutdownPkg 	#shutdown the package
	;;
	restart)
		echo "Restarting $PKG_NAME"
		$0 stop
		$0 start
	;;
	*)
		echo "Usage: $0 {start|stop|restart}"
esac

exit 0